@@ -4,6 +4,8 @@ module Test.Hspec.Core.Compat (
 , module Test.Hspec.Core.Compat
 ) where
 
+import           Control.Exception as Imports
+import           Control.Arrow as Imports ((>>>), (&&&), first, second)
 import           Control.Applicative as Imports
 import           Control.Monad as Imports hiding (
     mapM
@@ -14,7 +16,13 @@ import           Control.Monad as Imports hiding (
   , sequence
   , sequence_
   )
+import           Data.Maybe as Imports
 import           Data.Foldable as Imports
+import           Data.CallStack as Imports (HasCallStack)
+
+import           System.IO
+import           System.Exit
+import           System.Environment
 
 #if MIN_VERSION_base(4,11,0)
 import           Data.Functor as Imports
@@ -26,6 +34,7 @@ import           Data.List as Imports (
     stripPrefix
   , isPrefixOf
   , isInfixOf
+  , isSuffixOf
   , intercalate
   , inits
   , tails
@@ -69,7 +78,6 @@ import           Text.Read as Imports (readMaybe)
 import           System.Environment as Imports (lookupEnv)
 #else
 import           Text.Read
-import           System.Environment
 import qualified Text.ParserCombinators.ReadP as P
 #endif
 
@@ -77,16 +85,23 @@ import qualified Text.ParserCombinators.ReadP as P
 import           Data.Ord (comparing)
 #endif
 
+#if MIN_VERSION_base(4,7,0)
+import           Data.Bool as Imports (bool)
+#endif
+
 import           Data.Typeable (tyConModule, tyConName)
 import           Control.Concurrent
 
-#if MIN_VERSION_base(4,9,0)
-import           Control.Exception as Imports (interruptible)
-#else
+#if !MIN_VERSION_base(4,9,0)
 import           GHC.IO
 #endif
 
 #if !MIN_VERSION_base(4,6,0)
+forkFinally :: IO a -> (Either SomeException a -> IO ()) -> IO ThreadId
+forkFinally action and_then =
+  mask $ \restore ->
+    forkIO $ try (restore action) >>= and_then
+
 -- |Strict version of 'modifyIORef'
 modifyIORef' :: IORef a -> (a -> a) -> IO ()
 modifyIORef' ref f = do
@@ -97,7 +112,7 @@ modifyIORef' ref f = do
 atomicWriteIORef :: IORef a -> a -> IO ()
 atomicWriteIORef ref a = do
     x <- atomicModifyIORef ref (\_ -> (a, ()))
-    x `seq` return ()
+    x `seq` pass
 
 -- | Parse a string using the 'Read' instance.
 -- Succeeds if there is exactly one valid result.
@@ -159,8 +174,31 @@ sortOn f =
   map snd . sortBy (comparing fst) . map (\x -> let y = f x in y `seq` (y, x))
 #endif
 
+#if !MIN_VERSION_base(4,7,0)
+bool :: a -> a -> Bool -> a
+bool f _ False = f
+bool _ t True  = t
+#endif
+
 #if !MIN_VERSION_base(4,11,0)
 infixl 1 <&>
 (<&>) :: Functor f => f a -> (a -> b) -> f b
 (<&>) = flip (<$>)
 #endif
+
+endsWith :: Eq a => [a] -> [a] -> Bool
+endsWith = flip isSuffixOf
+
+#if MIN_VERSION_base(4,8,0)
+pass :: Applicative m => m ()
+pass = pure ()
+#else
+pass :: Monad m => m ()
+pass = return ()
+#endif
+
+die :: String -> IO a
+die err = do
+  name <- getProgName
+  hPutStrLn stderr $ name <> ": " <> err
+  exitFailure
