@@ -1,18 +1,18 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE TypeOperators #-}
+
 module Test.Hspec.Core.ExampleSpec (spec) where
 
+import           Prelude ()
 import           Helper
+
 import           Mock
-import           Control.Exception
 import           Test.HUnit (assertFailure, assertEqual)
 
-import           Test.Hspec.Core.Example (Result(..), ResultStatus(..)
-#if MIN_VERSION_base(4,8,1)
-  , Location(..)
-#endif
-  , FailureReason(..))
+import           Test.Hspec.Core.Example (Result(..), ResultStatus(..), FailureReason(..))
+import qualified Test.Hspec.Expectations as H
 import qualified Test.Hspec.Core.Example as H
 import qualified Test.Hspec.Core.Spec as H
 import qualified Test.Hspec.Core.Runner as H
@@ -29,34 +29,70 @@ evaluateExampleWith action e = H.evaluateExample e defaultParams (action . ($ ()
 evaluateExampleWithArgument :: H.Example e => (ActionWith (H.Arg e) -> IO ()) -> e -> IO Result
 evaluateExampleWithArgument action e = H.evaluateExample e defaultParams action noOpProgressCallback
 
+bottom :: a
+bottom = throw DivideByZero
+
 spec :: Spec
 spec = do
+  describe "safeEvaluate" $ do
+    let
+      status :: ResultStatus
+      status = Failure Nothing (Error Nothing $ toException DivideByZero)
+
+      err :: Result
+      err = Result "" status
+
+    it "forces Result" $ do
+      H.safeEvaluate (return $ Result "" bottom) `shouldReturn` err
+
+    it "handles ResultStatus exceptions" $ do
+      H.safeEvaluate (throwIO status) `shouldReturn` err
+
+    it "forces ResultStatus exceptions" $ do
+      H.safeEvaluate (throwIO $ Failure Nothing bottom) `shouldReturn` err
+
+    it "handles other exceptions" $ do
+      H.safeEvaluate (throwIO DivideByZero) `shouldReturn` err
+
+    it "forces other exceptions" $ do
+      H.safeEvaluate (throwIO $ ErrorCall bottom) `shouldReturn` err
+
+  describe "safeEvaluateResultStatus" $ do
+    let
+      err :: ResultStatus
+      err = Failure Nothing (Error Nothing $ toException DivideByZero)
+
+    it "forces ResultStatus" $ do
+      H.safeEvaluateResultStatus (return $ Failure Nothing bottom) `shouldReturn` err
+
+    it "handles ResultStatus exceptions" $ do
+      H.safeEvaluateResultStatus (throwIO err) `shouldReturn` err
+
+    it "forces ResultStatus exceptions" $ do
+      H.safeEvaluateResultStatus (throwIO $ Failure Nothing bottom) `shouldReturn` err
+
+    it "handles other exceptions" $ do
+      H.safeEvaluateResultStatus (throwIO DivideByZero) `shouldReturn` err
+
+    it "forces other exceptions" $ do
+      H.safeEvaluateResultStatus (throwIO $ ErrorCall bottom) `shouldReturn` err
+
   describe "safeEvaluateExample" $ do
     context "for Expectation" $ do
       it "returns Failure if an expectation does not hold" $ do
-        Result "" (Failure _ msg) <- safeEvaluateExample (23 `shouldBe` (42 :: Int))
+        Result "" (Failure _ msg) <- safeEvaluateExample (23 `H.shouldBe` (42 :: Int))
         msg `shouldBe` ExpectedButGot Nothing "42" "23"
 
       context "when used with `pending`" $ do
         it "returns Pending" $ do
           result <- safeEvaluateExample (H.pending)
-          let location =
-#if MIN_VERSION_base(4,8,1)
-                Just $ Location __FILE__ (__LINE__ - 3) 42
-#else
-                Nothing
-#endif
+          let location = mkLocation __FILE__ (pred __LINE__) 42
           result `shouldBe` Result "" (Pending location Nothing)
 
       context "when used with `pendingWith`" $ do
         it "includes the optional reason" $ do
           result <- safeEvaluateExample (H.pendingWith "foo")
-          let location =
-#if MIN_VERSION_base(4,8,1)
-                Just $ Location __FILE__ (__LINE__ - 3) 42
-#else
-                Nothing
-#endif
+          let location = mkLocation __FILE__ (pred __LINE__) 42
           result `shouldBe` Result "" (Pending location $ Just "foo")
 
   describe "evaluateExample" $ do
@@ -140,7 +176,7 @@ spec = do
 
       it "returns Failure if property does not hold" $ do
         Result "" (Failure _ _) <- evaluateExample $ property $ \n -> n /= (n :: Int)
-        return ()
+        pass
 
       it "shows what falsified it" $ do
         Result "" (Failure _ r) <- evaluateExample $ property $ \ (x :: Int) (y :: Int) -> (x == 0 && y == 1) ==> False
@@ -174,7 +210,7 @@ spec = do
         let prop p = property $ \ (x :: Int) (y :: Int) -> (x == 0 && y == 1) ==> p
         context "when used with shouldBe" $ do
           it "shows what falsified it" $ do
-            Result "" (Failure _ err) <- evaluateExample $ prop $ 23 `shouldBe` (42 :: Int)
+            Result "" (Failure _ err) <- evaluateExample $ prop $ 23 `H.shouldBe` (42 :: Int)
             err `shouldBe` ExpectedButGot (Just "Falsifiable (after 1 test):\n  0\n  1") "42" "23"
 
         context "when used with assertEqual" $ do
@@ -189,42 +225,32 @@ spec = do
 
         context "when used with verbose" $ do
           it "includes verbose output" $ do
-            Result info (Failure _ err) <- evaluateExample $ verbose $ (`shouldBe` (23 :: Int))
+            Result info (Failure _ err) <- evaluateExample $ verbose $ (`H.shouldBe` (23 :: Int))
             info `shouldBe` "Failed:\n0"
             err `shouldBe` ExpectedButGot (Just "Falsifiable (after 1 test):\n  0") "23" "0"
 
       context "when used with `pending`" $ do
         it "returns Pending" $ do
-          let location =
-#if MIN_VERSION_base(4,8,1)
-                Just $ Location __FILE__ (__LINE__ + 4) 37
-#else
-                Nothing
-#endif
+          let location = mkLocation __FILE__ (succ __LINE__) 37
           evaluateExample (property H.pending) `shouldReturn` Result "" (Pending location Nothing)
 
       context "when used with `pendingWith`" $ do
         it "includes the optional reason" $ do
-          let location =
-#if MIN_VERSION_base(4,8,1)
-                Just $ Location __FILE__ (__LINE__ + 4) 39
-#else
-                Nothing
-#endif
+          let location = mkLocation __FILE__ (succ __LINE__) 39
           evaluateExample (property $ H.pendingWith "foo") `shouldReturn` Result "" (Pending location $ Just "foo")
 
   describe "Expectation" $ do
     context "as a QuickCheck property" $ do
       it "can be quantified" $ do
         e <- newMock
-        silence . H.hspec $ do
+        hspecSilent $ do
           H.it "some behavior" $ property $ \xs -> do
             mockAction e
             (reverse . reverse) xs `shouldBe` (xs :: [Int])
         mockCounter e `shouldReturn` 100
 
       it "can be used with expectations/HUnit assertions" $ do
-        silence . H.hspecResult $ do
+        hspecResultSilent $ do
           H.describe "readIO" $ do
             H.it "is inverse to show" $ property $ \x -> do
               (readIO . show) x `shouldReturn` (x :: Int)
