@@ -6,18 +6,36 @@ import           Helper
 
 import           Test.Hspec.Core.Spec (Item(..), Result(..), ResultStatus(..))
 import qualified Test.Hspec.Core.Runner as H
-import           Test.Hspec.Core.Spec (Tree(..), runSpecM)
+import           Test.Hspec.Core.Spec (Tree(..))
 
 import qualified Test.Hspec.Core.Spec as H
 
 extract :: (Item () -> a) -> H.Spec -> IO [Tree () a]
-extract f = fmap (H.bimapForest (const ()) f) . runSpecM
+extract f = fmap (H.bimapForest (const ()) f) . fmap snd . H.runSpecM
 
 runSpec :: H.Spec -> IO [String]
 runSpec = captureLines . H.hspecResult
 
 spec :: Spec
 spec = do
+  let
+    runSpecM :: H.SpecWith a -> IO [H.SpecTree a]
+    runSpecM = fmap snd . H.runSpecM
+
+    runItem :: Item () -> IO Result
+    runItem item = itemExample item defaultParams ($ ()) noOpProgressCallback
+
+  describe "getSpecDescriptionPath" $ do
+    it "returns the spec path" $ do
+      let descriptionPathShouldBe xs =
+            H.getSpecDescriptionPath >>= H.runIO . (`shouldBe` xs)
+      void . runSpecM $ do
+        descriptionPathShouldBe []
+        H.describe "foo" $ do
+          H.describe "bar" $ do
+            descriptionPathShouldBe ["foo", "bar"]
+            H.it "baz" True
+
   describe "describe" $ do
     it "can be nested" $ do
       [Node foo [Node bar [Leaf _]]] <- runSpecM $ do
@@ -38,8 +56,7 @@ spec = do
   describe "xdescribe" $ do
     it "creates a tree of pending spec items" $ do
       [Node _ [Leaf item]] <- runSpecM (H.xdescribe "" $ H.it "whatever" True)
-      r <- itemExample item defaultParams ($ ()) noOpProgressCallback
-      r `shouldBe` Result "" (Pending Nothing Nothing)
+      runItem item `shouldReturn` Result "" (Pending Nothing Nothing)
 
   describe "it" $ do
     it "takes a description of a desired behavior" $ do
@@ -48,17 +65,11 @@ spec = do
 
     it "takes an example of that behavior" $ do
       [Leaf item] <- runSpecM (H.it "whatever" True)
-      r <- itemExample item defaultParams ($ ()) noOpProgressCallback
-      r `shouldBe` Result "" Success
+      runItem item `shouldReturn` Result "" Success
 
     it "adds source locations" $ do
       [Leaf item] <- runSpecM (H.it "foo" True)
-      let location =
-#if MIN_VERSION_base(4,8,1)
-            Just $ H.Location __FILE__ (__LINE__ - 3) 32
-#else
-            Nothing
-#endif
+      let location = mkLocation __FILE__ (pred __LINE__) 32
       itemLocation item `shouldBe` location
 
     context "when no description is given" $ do
@@ -73,8 +84,7 @@ spec = do
   describe "xit" $ do
     it "creates a pending spec item" $ do
       [Leaf item] <- runSpecM (H.xit "whatever" True)
-      r <- itemExample item defaultParams ($ ()) noOpProgressCallback
-      r `shouldBe` Result "" (Pending Nothing Nothing)
+      runItem item `shouldReturn` Result "" (Pending Nothing Nothing)
 
   describe "pending" $ do
     it "specifies a pending example" $ do
