@@ -1,8 +1,9 @@
 module Test.Hspec.Core.UtilSpec (spec) where
 
+import           Prelude ()
 import           Helper
+
 import           Control.Concurrent
-import qualified Control.Exception as E
 
 import           Test.Hspec.Core.Util
 
@@ -20,12 +21,12 @@ spec = do
 
   describe "formatException" $ do
     it "converts exception to string" $ do
-      formatException (E.toException E.DivideByZero) `shouldBe` "ArithException\ndivide by zero"
+      formatException (toException DivideByZero) `shouldBe` "ArithException\ndivide by zero"
 
     context "when used with an IOException" $ do
       it "includes the IOErrorType" $ do
         inTempDirectory $ do
-          Left e <- E.try (readFile "foo")
+          Left e <- try (readFile "foo")
           formatException e `shouldBe` intercalate "\n" [
               "IOException of type NoSuchThing"
             , "foo: openFile: does not exist (No such file or directory)"
@@ -42,6 +43,14 @@ spec = do
         , "sed do eiusmod"
         ]
 
+    it "preserves existing line breaks" $ do
+      lineBreaksAt 10 "foo bar baz\none two three" `shouldBe` [
+          "foo bar"
+        , "baz"
+        , "one two"
+        , "three"
+        ]
+
   describe "safeTry" $ do
     it "returns Right on success" $ do
       Right e <- safeTry (return 23 :: IO Int)
@@ -49,21 +58,21 @@ spec = do
 
     it "returns Left on exception" $ do
       Left e <- safeTry throwException
-      show e `shouldBe` "foobar"
+      fromException e `shouldBe` Just DivideByZero
 
     it "evaluates result to weak head normal form" $ do
-      Left e <- safeTry (return $ E.throw $ E.ErrorCall "foo")
+      Left e <- safeTry (return $ throw $ ErrorCall "foo")
       show e `shouldBe` "foo"
 
     it "does not catch asynchronous exceptions" $ do
       mvar <- newEmptyMVar
       sync <- newEmptyMVar
       threadId <- forkIO $ do
-        safeTry (putMVar sync () >> threadDelay 1000000) >> return ()
-        `E.catch` putMVar mvar
+        safeTry (putMVar sync () >> threadDelay 1000000) >> pass
+        `catch` putMVar mvar
       takeMVar sync
-      throwTo threadId E.UserInterrupt
-      readMVar mvar `shouldReturn` E.UserInterrupt
+      throwTo threadId UserInterrupt
+      readMVar mvar `shouldReturn` UserInterrupt
 
   describe "filterPredicate" $ do
     it "tries to match a pattern against a path" $ do
@@ -93,7 +102,7 @@ spec = do
     it "creates a sentence from a subject and a requirement" $ do
       formatRequirement (["reverse"], "reverses a list") `shouldBe` "reverse reverses a list"
 
-    it "creates a sentence from a subject and a requirement when the subject consits of multiple words" $ do
+    it "creates a sentence from a subject and a requirement when the subject consists of multiple words" $ do
       formatRequirement (["The reverse function"], "reverses a list") `shouldBe` "The reverse function reverses a list"
 
     it "returns the requirement if no subject is given" $ do
